/*=============================================================================
 * parser for CSP instances represented in XML format
 * 
 * Copyright (c) 2008 Olivier ROUSSEL (olivier.roussel <at> cril.univ-artois.fr)
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *=============================================================================
 */
#ifndef _C_CSPParserCallback_h_
#define _C_CSPParserCallback_h_

#include "C_AST.h"

#ifdef __cplusplus
extern "C"
{
#endif
  

/**
 * calls the parser with the name of the file to parse
 *
 * @param filename: the name of the file to parse
 * @param preferredSyntax: chooses the way the parser will transmit an
 * expression to the solver: (TREE=>the parser will transmit an
 * abstract syntax tree, PREFIX=> the parser will transmit a string
 * with an prefix representation, INFIX_C=> the parser will transmit a
 * string with a C expression, POSTFIX=> the parser will transmit a
 * string with a postfix expression, MATHML=> the parser will transmit
 * a string with a MathML expression)
 * @param solverData: data passed by the solver to each callback
 * functions (if you don't need it, set it to NULL)
 */

void parseXML(char *filename, 
	      Syntax preferredSyntax,
	      void *solverData);


/**
 * frees the tree pointed by root
 */
void freeC_AST(C_AST *root);

/**
 * callback interface for a C solver
 *
 *
 * see the C++ version (CSPParserCallback.hh) for a definition of
 * functions and parameters
 *
 * all strings are owned by the parser and should be copied when
 * necessary (strdup())
 *
 * solverData is a pointer that is passed by the solver to the
 * callback functions
 *
 */

void beginInstance(void *solverData, const char *name);;

void beginDomainsSection(void *solverData, int nbDomains); 

void beginDomain(void *solverData, const char *name, int idDomain, int nbValue); 

void addDomainValue(void *solverData, int v); 

void addDomainValueInterval(void *solverData, int first,int last); 

void endDomain(void *solverData); 

void endDomainsSection(void *solverData); 

void beginVariablesSection(void *solverData, int nbVariables); 

void addVariable(void *solverData, const char *name, int idVar,
		 const char *domain, int idDomain); 

void endVariablesSection(void *solverData); 

void beginRelationsSection(void *solverData, int nbRelations); 

void beginRelation(void *solverData, const char *name, int idRel,
		   int arity, int nbTuples, RelType relType); 

void addRelationTuple(void *solverData, int arity, int tuple[]); 

void addRelationWeightedTuple(void *solverData, int arity, int tuple[], int cost); 

void endRelation(void *solverData); 

void endRelationsSection(void *solverData); 

void beginPredicatesSection(void *solverData, int nbPredicates); 

void beginPredicate(void *solverData, const char *name, int idPred); 

void addFormalParameter(void *solverData, int pos, const char *name, const char *type); 

void predicateExpressionAST(void *solverData, void *tree); 

void predicateExpression(void *solverData, const char *expr); 

void endPredicate(void *solverData); 

void endPredicatesSection(void *solverData); 

void beginConstraintsSection(void *solverData, int nbConstraints); 

void beginConstraint(void *solverData, const char *name, int idConstr,
		     int arity, 
		     const char *reference,
		     int type, int id); 

void addVariableToScope(void *solverData, const char *name, int idVar); 

void constraintParameters(C_AST *args);

void endConstraint(void *solverData); 

void endConstraintsSection(void *solverData); 

void endInstance(void *solverData); 

#ifdef __cplusplus
}
#endif

#endif
