/*=============================================================================
 * parser for CSP instances represented in XML format
 * 
 * Copyright (c) 2008 Olivier ROUSSEL (olivier.roussel <at> cril.univ-artois.fr)
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *=============================================================================
 */
#include "XMLParser_libxml2.hh"
#include "C_CSPParserCallback.h"

using namespace CSPXMLParser;

class C_Callback
{
private:
  void *solverData; // data passed by the solver to the callbacks
public:
  C_Callback(void *solverData)
  {
    this->solverData=solverData;
  }

  void beginInstance(const string & name) 
  {
    ::beginInstance(solverData,name.c_str());
  }

  void beginDomainsSection(int nbDomains)  
  {
    ::beginDomainsSection(solverData,nbDomains);
  }
  
  void beginDomain(const string & name, int idDomain, int nbValue)  
  {
    ::beginDomain(solverData,name.c_str(),idDomain,nbValue);
  }

  void addDomainValue(int v)  
  {
    ::addDomainValue(solverData,v);
  }

  void addDomainValue(int first,int last)  
  {
    ::addDomainValueInterval(solverData,first,last);
  }

  void endDomain()  
  {
    ::endDomain(solverData);
  }

  void endDomainsSection()  
  {
    ::endDomainsSection(solverData);
  }

  void beginVariablesSection(int nbVariables)  
  {
    ::beginVariablesSection(solverData,nbVariables);
  }
  
  void addVariable(const string & name, int idVar, 
			   const string & domain, int idDomain)  
  {
    ::addVariable(solverData,name.c_str(),idVar,domain.c_str(),idDomain);
  }

  void endVariablesSection()  
  {
    ::endVariablesSection(solverData);
  }


  void beginRelationsSection(int nbRelations)  
  {
    ::beginRelationsSection(solverData,nbRelations);
  }
  
  void beginRelation(const string & name, int idRel,
			     int arity, int nbTuples, RelType relType)  
  {
    ::beginRelation(solverData,name.c_str(),idRel,arity,nbTuples,relType);
  }

  void addRelationTuple(int arity, int tuple[])  
  {
    ::addRelationTuple(solverData,arity,tuple);
  }

  void addRelationTuple(int arity, int tuple[],int cost)  
  {
    ::addRelationWeightedTuple(solverData,arity,tuple,cost);
  }

  void endRelation()  
  {
    ::endRelation(solverData);
  }

  void endRelationsSection()  
  {
    ::endRelationsSection(solverData);
  }

  void beginPredicatesSection(int nbPredicates)  
  {
    ::beginPredicatesSection(solverData,nbPredicates);
  }
  
  void beginPredicate(const string & name, int idPred)  
  {
    ::beginPredicate(solverData,name.c_str(),idPred);
  }

  void addFormalParameter(int pos, const string & name, const string & type)  
  {
    ::addFormalParameter(solverData,pos,name.c_str(),type.c_str());
  }

  void predicateExpression(AST *tree)  
  {
    ::predicateExpressionAST(solverData,NULL);
  }

  void predicateExpression(const string &expr)  
  {
    ::predicateExpression(solverData,expr.c_str()); 
  }

  void endPredicate()  
  {
    ::endPredicate(solverData);
  }

  void endPredicatesSection()  
  {
    ::endPredicatesSection(solverData);
  }

  void beginConstraintsSection(int nbConstraints)  
  {
    ::beginConstraintsSection(solverData,nbConstraints);
  }
  
  void beginConstraint(const string & name, int idConstr,
		       int arity, const string & reference, 
		       CSPDefinitionType type, int id,
		       const ASTList &scope)  
  {
    ::beginConstraint(solverData,name.c_str(),idConstr,arity,reference.c_str(),type,id);

    for(int i=0;i<scope.size();++i)
      ::addVariableToScope(solverData,
			   scope[i].getVarName().c_str(),
			   scope[i].getVarId());
  }

  void constraintParameters(const ASTList &args)
  {
    ::constraintParameters(args.makeCTree());
  }

  void endConstraint()  
  {
    ::endConstraint(solverData);
  }

  void endConstraintsSection()  
  {
    ::endConstraintsSection(solverData);
  }

  void endInstance()  
  {
    ::endInstance(solverData);
  }
};

void parseXML(char *filename, Syntax preferredSyntax, 
	      void *solverData)
{
  C_Callback cb(solverData);

  try
  {
    XMLParser_libxml2<C_Callback> parser(cb);

    parser.setPreferredExpressionRepresentation(preferredSyntax);

    parser.parse(filename);
  }
  catch (exception &e)
  {
    cout.flush();
    cerr << "\n\tUnexpected exception :\n";
    cerr << "\t" << e.what() << endl;
    exit(1);
  }
}

void freeC_AST(C_AST *node)
{
  int i;
  C_AST_VarNode *var;
  C_AST_FxNode *fx;
  C_AST_ListNode *list;
  C_AST_DictNode *dict;

  switch(node->type)
  {
  case VAR:
    var=(C_AST_VarNode *)node;
    free(var->varName);
    break;

  case CST_INT:
  case CST_BOOL:
    break;

  case F_NEG:
  case F_ABS:
  case F_ADD:
  case F_SUB:
  case F_MUL:
  case F_DIV:
  case F_MOD:
  case F_POW:
  case F_IF:
  case F_MIN:
  case F_MAX:
  case F_EQ:
  case F_NE:
  case F_GE:
  case F_GT:
  case F_LE:
  case F_LT:
  case F_NOT:
  case F_AND:
  case F_OR:
  case F_XOR:
  case F_IFF:
    fx=(C_AST_FxNode*)node;
    for(i=0;i<fx->nbarg;++i)
      freeC_AST(fx->args[i]);
    break;

  case LIST:
    list=(C_AST_ListNode*)node;
    for(i=0;i<list->size;++i)
      freeC_AST(list->items[i]);
    break;

  case DICT:
    dict=(C_AST_DictNode*)node;
    for(i=0;i<dict->size;++i)
    {
      free(dict->items[i].key);
      freeC_AST(dict->items[i].value);
    }
    break;

  case SYMB_EQ:
  case SYMB_NE:
  case SYMB_GE:
  case SYMB_GT:
  case SYMB_LE:
  case SYMB_LT:
    break;

  default:
    printf("oops! don't know how to free this node in C_AST\n");
  }

  free(node);
}


