/*=============================================================================
 * parser for CSP instances represented in XML format
 * 
 * Copyright (c) 2008 Olivier ROUSSEL (olivier.roussel <at> cril.univ-artois.fr)
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *=============================================================================
 */
#include <stdio.h>
#include <stdlib.h>

#include "C_CSPParserCallback.h"

void printC_AST_FxNode(C_AST *node, char *fxname);

void printC_AST(C_AST *node)
{
  int i;
  C_AST_VarNode *var;
  C_AST_CstNode *cst;
  C_AST_ListNode *list;
  C_AST_DictNode *dict;

  switch(node->type)
  {
  case VAR:
    var=(C_AST_VarNode *)node;
    printf("%s/$%d ",var->varName,var->idVar);
    break;

  case CST_INT:
  case CST_BOOL:
    cst=(C_AST_CstNode *)node;
    printf("%d ",cst->val);
    break;

  case F_NEG:
    printC_AST_FxNode(node,"neg");
    break;

  case F_ABS:
    printC_AST_FxNode(node,"abs");
    break;
 
  case F_ADD:
    printC_AST_FxNode(node,"add");
    break;
 
  case F_SUB:
    printC_AST_FxNode(node,"sub");
    break;
 
  case F_MUL:
    printC_AST_FxNode(node,"mul");
    break;
 
  case F_DIV:
    printC_AST_FxNode(node,"div");
    break;
 
  case F_MOD:
    printC_AST_FxNode(node,"mod");
    break;
 
  case F_POW:
    printC_AST_FxNode(node,"pow");
    break;
 
  case F_IF:
    printC_AST_FxNode(node,"if");
    break;
 
  case F_MIN:
    printC_AST_FxNode(node,"min");
    break;
 
  case F_MAX:
    printC_AST_FxNode(node,"max");
    break;
 
  case F_EQ:
    printC_AST_FxNode(node,"eq");
    break;
 
  case F_NE:
    printC_AST_FxNode(node,"ne");
    break;
 
  case F_GE:
    printC_AST_FxNode(node,"ge");
    break;
 
  case F_GT:
    printC_AST_FxNode(node,"gt");
    break;
 
  case F_LE:
    printC_AST_FxNode(node,"le");
    break;
 
  case F_LT:
    printC_AST_FxNode(node,"lt");
    break;
 
  case F_NOT:
    printC_AST_FxNode(node,"not");
    break;
 
  case F_AND:
    printC_AST_FxNode(node,"and");
    break;
 
  case F_OR:
    printC_AST_FxNode(node,"or");
    break;
 
  case F_XOR:
    printC_AST_FxNode(node,"xor");
    break;

  case F_IFF:
    printC_AST_FxNode(node,"iff");
    break;
 
  case LIST:
    list=(C_AST_ListNode*)node;
    printf("[");
    for(i=0;i<list->size;++i)
      printC_AST(list->items[i]);
    printf("] ");
    break;

  case DICT:
    dict=(C_AST_DictNode*)node;
    printf("{");
    for(i=0;i<dict->size;++i)
    {
      printf("/%s ",dict->items[i].key);
      printC_AST(dict->items[i].value);
    }
    printf("} ");
    break;

  case SYMB_EQ:
    printf("== ");
    break;

  case SYMB_NE:
    printf("!= ");
    break;

  case SYMB_GE:
    printf(">= ");
    break;

  case SYMB_GT:
    printf("> ");
    break;

  case SYMB_LE:
    printf("<= ");
    break;

  case SYMB_LT:
    printf("< ");
    break;

  default:
    printf("? ");
  }
}

void printC_AST_FxNode(C_AST *node, char *fxname)
{
  C_AST_FxNode *fx;
  int i;
  
  fx=(C_AST_FxNode*)node;
  printf("%s(",fxname);
  printC_AST(fx->args[0]);
  for(i=1;i<fx->nbarg;++i)
  {
    printf(",");
    printC_AST(fx->args[i]);
  }

  printf(") ");
}

void beginInstance(void *solverData, const char *name)
{
  printf("beginInstance(name=\"%s\")\n",name);
}

void beginDomainsSection(void *solverData, int nbDomains) 
{  
  printf("beginDomainsSection(nbDomains=%d)\n",nbDomains);
}

void beginDomain(void *solverData, const char *name, int idDomain, int nbValue) 
{
  printf("  beginDomain(name=\"%s\",idDomain=%d,nbValue=%d)\n",name,idDomain,nbValue);
}

void addDomainValue(void *solverData, int v) 
{
  printf("    addDomainValue(v=%d)\n",v);
}

void addDomainValueInterval(void *solverData, int first,int last) 
{
  printf("    addDomainValueInterval(first=%d,last=%d)\n",first,last);
}

void endDomain(void *solverData) 
{
  printf("  endDomain()\n");
}

void endDomainsSection(void *solverData) 
{
  printf("endDomainsSection()\n");
}


void beginVariablesSection(void *solverData, int nbVariables) 
{
  printf("beginVariablesSection(nbVariables=%d)\n",nbVariables);
}

void addVariable(void *solverData, const char *name, int idVar,
		 const char *domain, int idDomain) 
{
  printf("  addVariable(name=\"%s\",idVar=%d,domain=\"%s\",idDomain=%d)\n",
	 name,idVar,domain,idDomain);
}

void endVariablesSection(void *solverData) 
{
  printf("endVariablesSection()\n");
}


void beginRelationsSection(void *solverData, int nbRelations) 
{
  printf("beginRelationsSection(nbRelations=%d)\n",nbRelations);
}


void beginRelation(void *solverData, const char *name, int idRel,
		   int arity, int nbTuples, RelType relType) 
{
  printf("  beginRelation(name=\"%s\",idRel=%d,arity=%d,nbTuples=%d,semantics=",
	 name,idRel,arity,nbTuples);

  switch(relType)
  {
  case REL_SUPPORT:
    printf("supports");
    break;
  case REL_CONFLICT:
    printf("conflicts");
    break;
  case REL_SOFT:
    printf("soft");
    break;
  default:
    printf("?");
  }

  printf(")\n");
}

void addRelationTuple(void *solverData, int arity, int tuple[]) 
{
  int i;
  printf("    addRelationTuple(arity=%d,tuple=[%d",arity,tuple[0]);
  for(i=1;i<arity;++i)
    printf(",%d",tuple[i]);
  printf("])\n");
}

void addRelationWeightedTuple(void *solverData, int arity, int tuple[], int cost) 
{
  int i;
  printf("    addRelationWeightedTuple(arity=%d,tuple=[%d",arity,tuple[0]);
  for(i=1;i<arity;++i)
    printf(",%d",tuple[i]);
  printf("],cost=%d)\n",cost);
}

void endRelation(void *solverData) 
{
  printf("  endRelation()\n");
}

void endRelationsSection(void *solverData) 
{
  printf("endRelationsSection()\n");
}

void beginPredicatesSection(void *solverData, int nbPredicates) 
{
  printf("beginPredicatesSection(nbPredicates=%d)\n",nbPredicates);
}

void beginPredicate(void *solverData, const char *name, int idPred) 
{
  printf("  beginPredicate(name=\"%s\",idPred=%d)\n",name,idPred);
}

void addFormalParameter(void *solverData, int pos, const char *name, const char *type) 
{
  printf("    addFormalParameter(pos=%d,name=\"%s\",type=\"%s\")\n",
	 pos,name,type);
}

void predicateExpressionAST(void *solverData, void *tree) 
{
  printf("    predicateExpressionAST(C_AST *tree)\n");
}

void predicateExpression(void *solverData, const char *expr) 
{
  printf("    predicateExpression(expr=\"%s\")\n",expr);
}

void endPredicate(void *solverData) 
{
  printf("  endPredicate()\n");
}

void endPredicatesSection(void *solverData) 
{
  printf("endPredicatesSection()\n");
}

void beginConstraintsSection(void *solverData, int nbConstraints) 
{
  printf("beginConstraintsSection(nbConstraints=%d)\n",nbConstraints);
}

void beginConstraint(void *solverData, const char *name, int idConstr,
		     int arity, 
		     const char *reference, 
		     int type, int id) 
{
  printf("  beginConstraint(name=\"%s\",idConstr=%d,arity=%d,reference=\"%s\",type=%d,id=%d)\n",
	 name,idConstr,arity,reference,type,id);
}

void addVariableToScope(void *solverData, const char *name, int idVar) 
{
  printf("    addVariableToScope(name=\"%s\",idVar=%d)\n",name,idVar);
}

void constraintParameters(C_AST *args)
{
  printf("    constraint parameters=");
  printC_AST(args);
  freeC_AST(args);
  printf("\n");
}

void endConstraint(void *solverData) 
{
  printf("  endConstraint()\n");
}

void endConstraintsSection(void *solverData) 
{
  printf("endConstraintsSection()\n");
}

void endInstance(void *solverData) 
{
  printf("endInstance()\n");
}


int main(int argc, char **argv)
{
  parseXML(argv[1],POSTFIX,NULL);

  return 0;
}
