/*=============================================================================
 * parser for CSP instances represented in XML format
 * 
 * Copyright (c) 2008 Olivier ROUSSEL (olivier.roussel <at> cril.univ-artois.fr)
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *=============================================================================
 */
#include <iostream>
#include <vector>
#include <getopt.h>

#include "CSPVerifierCallback.hh"
#include "XMLParser_libxml2.hh"

using namespace CSPXMLParser;
using namespace std;

/**
 * read a line from the solver output (available from file f), skip
 * comments (lines starting with "c ") and store the first non comment
 * line in s
 */
void skipCommentsAndReadOneLine(istream &f, string &s)
{
  do
  {
    getline(f,s);
  }
  while(f.good() && s.substr(0,2)=="c ");
}


/**
 * read the interpretation
 */ 
void readInterpretation(istream &f, CSPVerifierCallback &cb)
{
  char c;
  int i,val;
  bool gotEOL=false;

  cb.clearVarValues();

  i=0;
  while(true)
  {
    c=f.peek();
    if (!f.good())
      break;

    if (c=='\r' || c=='\n')
    {
      gotEOL=true;
      break;
    }

    if (isspace(c))
    {
      f.get();
      continue;
    }

    f >> val;
    if (!f.good())
      break;

    cb.setVarValue(i,val);
    ++i;
  } 

  if (i==0)
  {
    cout << "ERROR: no interpretation found !" << endl;
    exit(7);
  }

  if (!gotEOL)
  {
    cout << "ERROR: no interpretation found !" << endl;
    cout << "Interpretation is incomplete (doesn't end with a new line)" << endl;
    exit(7);
  }
}

/**
 * we expect a first line with SAT/UNSAT and then the certificate
 */
void readCSPSolutionFileSyntax(CSPVerifierCallback &verifier, 
			    string expectedAnswer)
{
  string answer;
 
  skipCommentsAndReadOneLine(cin,answer);

  if (answer=="UNSAT")
  {
    if (expectedAnswer.length())
      if (expectedAnswer=="UNSAT")
      {
	cout << "OK" << endl;
	exit(0);
      }
      else
      {
	cout << "ERROR: got UNSAT answer instead of expected SAT answer" << endl;
	exit(1);
      }

    cout << "No possible verification on an UNSAT instance" << endl;
    exit(0);
  }
  else
    if (answer!="SAT")
    {
      cout << "ERROR: Unexpected answer ! (SAT/UNSAT expected)" << endl;
      cout << "Got answer: " << answer << endl;
      exit(1);
    }

    readInterpretation(cin,verifier);
}

/**
 * we expect a 's ' line and a 'v ' line
 */
void readCSPStdoutSyntax(CSPVerifierCallback &verifier, 
		      string expectedAnswer)
{
  string answer,word;
  bool vLineFound=false;

  answer="<no 's ' line found>";

  while (cin.good())
  {
    cin >> word;
    if (!cin.good())
      break;

    if (word=="s")
    {
      cin >> answer;
      getline(cin,word);
    }
    else
      if (word=="v")
      {
	vLineFound=true;
	readInterpretation(cin,verifier);
      }
      else
	getline(cin,word);
  }

  if (answer=="UNSATISFIABLE")
  {
    if (expectedAnswer.length())
      if (expectedAnswer=="UNSAT")
      {
	cout << "OK" << endl;
	exit(0);
      }
      else
      {
	cout << "ERROR: got UNSATISFIABLE answer instead of expected SATISFIABLE answer" << endl;
	exit(1);
      }

    cout << "No possible verification on an UNSAT instance" << endl;
    exit(0);
  }
  else
    if (answer!="SATISFIABLE")
    {
      cout << "ERROR: Unexpected answer ! ('s SATISFIABLE'/'s UNSATISFIABLE' expected)" << endl;
      cout << "Got answer: " << answer << endl;
      exit(1);
    }

  if (!vLineFound)
  {
    cout << "ERROR: no interpretation found !" << endl;
    exit(7);
  }
}

/**
 * we look for the last valid assignment (last line with "prefix")
 *
 * prefix must be a single word with a trailing space ("SOL " or "v "
 * expected)
 */
void readLastMAXCSPAssignment(CSPVerifierCallback &verifier, 
			      string prefix)
{
  string line,assignment;

  while (cin.good())
  {
    getline(cin,line);
    if (!cin.good())
      break; // an incomplete line (without a '\n' at the end) will be
	     // ignored by this break

    if (line.substr(0,prefix.length())!=prefix)
      continue;
    cout << "line=" << line << endl;

    assignment=line;
  }

  cout << "assignment=" << assignment << endl;

  if (!assignment.length())
  {
    cout << "ERROR: no interpretation found !" << endl;
    exit(7);
  }

  assignment+='\n';

  istringstream input(assignment);
  input >> line; // skip the prefix (must be one word)
  readInterpretation(input,verifier);
}


struct option longopts[] =
{
  {"solutionfile", no_argument, NULL, 'S'},
  {"list", no_argument, NULL, 'l'},
  {NULL, no_argument, NULL, 0}
};


int main(int argc, char **argv)
{
  CSPVerifierCallback verifier;
  bool solutionFileSyntax=false;

  {
    char *prgname=argv[0],*s;

    s=strrchr(prgname,'/');
    if (s!=NULL)
      prgname=s+1;

    if (strcmp(prgname,"MCSP-verifier")==0)
      verifier.setMAXCSPmode();
  }

  int option;

  while ((option=getopt_long(argc,argv,"+",longopts,NULL))
	 !=EOF)
  {
    switch (option)
    {
    case 'S':
      solutionFileSyntax=true;
      break;
    case 'l':
      verifier.listAllUnsatisfiedConstraints();
      break;
    }
  }

  if (argc!=2 && argc!=3)
  {
    cout << "usage: " << argv[0] << " [--solutionfile] <filename> [<expected answer>]" << endl
	 << "\tverifies the solution of a CSP instance" << endl
	 << "\tthe solution is read on the standard input" << endl
	 << "\tformula and solution must conform to the CSP'06 "
      "evaluation formats" << endl;
    cout << "\texpected answer can be SAT or UNSAT" << endl;
    
    exit(1);
  }

  string expectedAnswer;

  if (argc>optind+1)
    expectedAnswer=argv[optind+1];

  try
  {
    XMLParser_libxml2<> parser(verifier);

    parser.setPreferredExpressionRepresentation(TREE);

    if (verifier.getMAXCSPmode())
    {
      if (solutionFileSyntax)
	readLastMAXCSPAssignment(verifier,"SOL ");
      else
	readLastMAXCSPAssignment(verifier,"v ");
    }
    else
    {
      if (solutionFileSyntax)
	readCSPSolutionFileSyntax(verifier,expectedAnswer);
      else
	readCSPStdoutSyntax(verifier,expectedAnswer);
    }

    int err=parser.parse(argv[optind]); // parse the input file and verify

    if (verifier.getMAXCSPmode() || verifier.listMode())
    {
      cout << verifier.getNbViolatedConstraints() 
	   << " unsatisfied constraints, "
	   << verifier.getNbSatisfiedConstraints() 
	   << " satisfied constraints"
	   << endl;

      exit(0);
    }

    // if the verifier callback didn't complain, then everything is OK
    if (err==0)
      cout << "OK" << endl;
    else
      cout << "ERROR: parser problems" << endl;
  }
  catch (exception &e)
  {
    cout.flush();
    cerr << "\n\tUnexpected exception :\n";
    cerr << "\t" << e.what() << endl;
    exit(1);
  }

  return 0;
}
